package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.ResultDetailDTO;
import com.diagbot.entity.BehospitalInfo;
import com.diagbot.entity.QcresultInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.FilterVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/4/13 16:45
 */
@Component
public class ConsoleFacade {

    @Autowired
    private QcresultInfoFacade qcresultInfoFacade;
    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;


    /**
     * 病历相关统计
     *
     * @return
     */
    public Map<String, Object> mrStatistics(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        retMap.put("本月病历数", 0);
        retMap.put("本月质控数-人工", 0);
        retMap.put("本月质控数-机器", 0);
        retMap.put("本月甲级病历-人工", 0);
        retMap.put("本月甲级病历-机器", 0);
        retMap.put("本月乙级病历-人工", 0);
        retMap.put("本月乙级病历-机器", 0);
        retMap.put("本月不合格病历-人工", 0);
        retMap.put("本月不合格病历-机器", 0);

        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = getStartDateStr(filterVO.getType());
        filterVO.setStartDate(startDate);
        filterVO.setHospitalId(hospitalId);

        QueryWrapper<BehospitalInfo> behospitalInfoQueryWrapper = new QueryWrapper<>();
        behospitalInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId)
                .ge("leave_hospital_date", startDate);
        List<BehospitalInfo> behospitalInfoList = behospitalInfoFacade.list(behospitalInfoQueryWrapper);
        if (ListUtil.isNotEmpty(behospitalInfoList)) {
            //本月病历数
            retMap.put("本月病历数", behospitalInfoList.size());
        }

        List<QcresultInfo> qcresultInfoList = qcresultInfoFacade.getQcresultSelectively(filterVO);
        if (ListUtil.isNotEmpty(qcresultInfoList)) {
            retMap.put("本月质控数-人工", qcresultInfoList
                    .stream()
                    .filter(i -> i.getGradeType().equals(2))
                    .collect(Collectors.toList())
                    .size());
            retMap.put("本月质控数-机器", qcresultInfoList
                    .stream()
                    .filter(i -> i.getGradeType().equals(1))
                    .collect(Collectors.toList())
                    .size());
            retMap.put("本月甲级病历-人工", qcresultInfoList
                    .stream()
                    .filter(i -> i.getGradeType().equals(2) && i.getLevel().equals("甲"))
                    .collect(Collectors.toList())
                    .size());
            retMap.put("本月甲级病历-机器", qcresultInfoList
                    .stream()
                    .filter(i -> i.getGradeType().equals(1) && i.getLevel().equals("甲"))
                    .collect(Collectors.toList())
                    .size());
            retMap.put("本月乙级病历-人工", qcresultInfoList
                    .stream()
                    .filter(i -> i.getGradeType().equals(2) && i.getLevel().equals("乙"))
                    .collect(Collectors.toList())
                    .size());
            retMap.put("本月乙级病历-机器", qcresultInfoList
                    .stream()
                    .filter(i -> i.getGradeType().equals(1) && i.getLevel().equals("乙"))
                    .collect(Collectors.toList())
                    .size());
            retMap.put("本月不合格病历-人工", qcresultInfoList
                    .stream()
                    .filter(i -> i.getGradeType().equals(2) && i.getLevel().equals("丙"))
                    .collect(Collectors.toList())
                    .size());
            retMap.put("本月不合格病历-机器", qcresultInfoList
                    .stream()
                    .filter(i -> i.getGradeType().equals(1) && i.getLevel().equals("丙"))
                    .collect(Collectors.toList())
                    .size());
        }

        return retMap;
    }

    /**
     * 缺陷相关统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> resultStatistics(FilterVO filterVO) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        retMap.put("缺陷排行列表", Lists.newLinkedList());
        retMap.put("各科室缺陷占比", Lists.newLinkedList());
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = getStartDateStr(filterVO.getType());
        filterVO.setStartDate(startDate);
        filterVO.setHospitalId(hospitalId);
        filterVO.setLimitCount(10);
        List<ResultDetailDTO> results = behospitalInfoFacade.resultStatistics(filterVO);
        if (ListUtil.isNotEmpty(results)) {
            results.forEach(result -> {
                DecimalFormat df = new DecimalFormat("#.00");
                String percentStr
                        = df.format(BigDecimal.valueOf(result.getPercent()).multiply(BigDecimal.valueOf(100))) + "%";
                result.setPercentStr(percentStr);
            });
            retMap.put("缺陷排行列表", results);
        }
        List<ResultDetailDTO> results2 = behospitalInfoFacade.resultStatisticsByDept(filterVO);
        if (ListUtil.isNotEmpty(results2)) {
            List<ResultDetailDTO> retResults = Lists.newLinkedList();
            if (results2.size() <= 6) {
                retResults = BeanUtil.listCopyTo(results2, ResultDetailDTO.class);
            } else {

                Double rate = 0d;
                Integer num = 0;
                for (ResultDetailDTO result : results2) {
                    if (retResults.size() < 5) {
                        rate = BigDecimal.valueOf(rate)
                                .add(BigDecimal.valueOf(Double.valueOf(result.getPercent())))
                                .doubleValue();
                        retResults.add(result);
                    } else {
                        num += result.getNum();

                    }
                }
                ResultDetailDTO retResult = new ResultDetailDTO();
                retResult.setName("其他");
                retResult.setNum(num);
                retResult.setPercent(BigDecimal.valueOf(1)
                        .min(BigDecimal.valueOf(rate))
                        .doubleValue());
                retResults.add(retResult);
            }
            retResults.forEach(result -> {
                DecimalFormat df = new DecimalFormat("#.00");
                String percentStr
                        = df.format(BigDecimal.valueOf(result.getPercent()).multiply(BigDecimal.valueOf(100))) + "%";
                result.setPercentStr(percentStr);
            });
            retMap.put("各科室缺陷占比", retResults);
        }
        return retMap;
    }

    /**
     * 筛选起始时间
     * @param type
     * @return
     */
    public String  getStartDateStr(Integer type) {
        Date date = new Date();
        String startDate = "";
        String year = DateUtil.getYear(date);
        int month = DateUtil.getMonth(date);
        if (type.equals(1)) {
            //本月统计
            startDate = year + "-" + month + "-1";
        } else if (type.equals(2)) {
            //本年统计
            startDate = year + "-1-1";
        }
        return startDate;
    }

}
